
import { createContext, useContext, useEffect, useState, FC } from 'react'
import { GoogleLoginResponse, useGoogleLogin, useGoogleLogout } from 'react-google-login'
import { useHistory } from 'react-router-dom'

import { Manager } from 'Manager'
import { ManagerService } from 'services'
import { tokenStorage } from 'utils'


const GoogleAuthContext = createContext<any>({});

export const useGoogleAuth = () => useContext(GoogleAuthContext);
export default useGoogleAuth;

export const GoogleAuthProvider: FC = ({ children }) => {
    const { push: historyPush } = useHistory();
    const [isSignedIn, setIsSignedIn] = useState(false)
    const [staySignedIn, setStaySignedIn] = useState(true)
    const [manager, setManager] = useState<Manager | null>()
    const [authError, setAuthError] = useState<string | null>()
    const [accessToken, setAccessToken] = useState<string | null>()
    const { signIn, loaded } = useGoogleLogin({ 
        clientId: (process.env.REACT_APP_GOOGLE_CLIENT_ID as string),
        onSuccess: (res: any) => {
            const {
                profileObj: {
                    email, 
                    familyName: lastName, 
                    givenName: firstName, 
                    googleId, 
                    imageUrl
                },
                tokenId: token
            } = res;

            const managerData: Manager = {
                email,
                firstName,
                lastName,
                googleId,
                imageUrl
            }
       
            setManager(managerData)
            setAccessToken(token)
            console.log(token)

            ManagerService.authenticate(token)
                .then(() => {
                    setIsSignedIn(true)
                    historyPush("/orders")
                })
 
         },
        onFailure: (err) =>  { 
            console.error("GOOGLE AUTH SIGN-IN ERROR: ", err) 
            setAuthError(`Ooops. Something went wrong: ${err?.message}`)
        },
        isSignedIn: staySignedIn,
        cookiePolicy: 'single_host_origin'
    })

    const { signOut } = useGoogleLogout({ 
        clientId: (process.env.REACT_APP_GOOGLE_CLIENT_ID as string),
        onLogoutSuccess: () =>  { 
            setIsSignedIn(false)
            historyPush("/") 
        },
        onFailure: () =>  { 
            console.error("A GOOGLE AUTH SIGN-OUT ERROR OCCURED") 
            setAuthError("Ooops. Something went wrong")
        },
     })

     useEffect(() => {
        if (accessToken) tokenStorage.set(accessToken);
     }, [accessToken])

    const authContextValues = {
        accessToken,
        authError,
        isSignedIn,
        loaded, 
        setStaySignedIn,
        signIn,
        signOut,
        staySignedIn,
        manager
    };

  return <GoogleAuthContext.Provider value={authContextValues}>{children}</GoogleAuthContext.Provider>
}
